
RPL_WELCOME     = "001"
RPL_ISUPPORT    = "005"
RPL_MOTD        = "372"
RPL_MOTDSTART   = "375"
RPL_UMODEIS     = "221"
RPL_VISIBLEHOST = "396"
RPL_TRYAGAIN    = "263"

ERR_NOSUCHNICK   = "401"
ERR_NOSUCHSERVER = "402"

RPL_CHANNELMODEIS = "324"
RPL_CREATIONTIME  = "329"
RPL_TOPIC         = "332"
RPL_TOPICWHOTIME  = "333"

RPL_WHOREPLY   = "352"
RPL_WHOSPCRPL  = "354"
RPL_ENDOFWHO   = "315"
RPL_NAMREPLY   = "353"
RPL_ENDOFNAMES = "366"

RPL_WHOWASUSER  = "314"
RPL_ENDOFWHOWAS = "369"

RPL_BANLIST        = "367"
RPL_ENDOFBANLIST   = "368"
RPL_QUIETLIST      = "728"
RPL_ENDOFQUIETLIST = "729"

RPL_LOGGEDIN    = "900"
RPL_LOGGEDOUT   = "901"
RPL_SASLSUCCESS = "903"
ERR_SASLFAIL    = "904"
ERR_SASLTOOLONG = "905"
ERR_SASLABORTED = "906"
ERR_SASLALREADY = "907"
RPL_SASLMECHS   = "908"

RPL_WHOISUSER     = "311"
RPL_WHOISSERVER   = "312"
RPL_WHOISOPERATOR = "313"
RPL_WHOISIDLE     = "317"
RPL_WHOISCHANNELS = "319"
RPL_WHOISACCOUNT  = "330"
RPL_WHOISHOST     = "378"
RPL_WHOISMODES    = "379"
RPL_WHOISSECURE   = "671"
RPL_AWAY          = "301"
RPL_ENDOFWHOIS    = "318"

ERR_ERRONEUSNICKNAME = "432"
ERR_NICKNAMEINUSE    = "433"
ERR_BANNICKCHANGE    = "435"
ERR_UNAVAILRESOURCE  = "437"
ERR_NICKTOOFAST      = "438"
ERR_CANTCHANGENICK   = "447"

ERR_NOSUCHCHANNEL   = "403"
ERR_TOOMANYCHANNELS = "405"
ERR_USERONCHANNEL   = "443"
ERR_LINKCHANNEL     = "470"
ERR_BADCHANNAME     = "479"
ERR_BADCHANNEL      = "926"


ERR_BANNEDFROMCHAN = "474"
ERR_INVITEONLYCHAN = "473"
ERR_BADCHANNELKEY  = "475"
ERR_CHANNELISFULL  = "471"
ERR_NEEDREGGEDNICK = "477"
ERR_THROTTLE       = "480"
